﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;

using MindFusion.Diagramming;
using MindFusion.Diagramming.Lanes;

using Pen = MindFusion.Drawing.Pen;
using SolidBrush = MindFusion.Drawing.SolidBrush;
using GradientBrush = MindFusion.Drawing.LinearGradientBrush;


public partial class Default2 : System.Web.UI.Page
{
	protected void Page_Load(object sender, EventArgs e)
	{
		if (!IsPostBack)
		{
			Diagram diagram = diagramView.Diagram;

			var headerFont = new Font("Trebuchet MS", 8.5f);

			var dashPen = new Pen(Color.FromArgb(230, 231, 233), 0);
			dashPen.DashStyle = DashStyle.Dash;
			var borderPen = new Pen(Color.FromArgb(210, 211, 213), 0);
			var whitePen = new Pen(Color.White, 0);
			var primaryColor = Color.FromArgb(0x94, 0xA5, 0xBD);
			var secondaryColor = Color.FromArgb(0x39, 0x39, 0x50);

			// Initialize grid
			Grid grid = diagram.LaneGrid;

			grid.HookHeaders = true;
			grid.HeadersOnTop = true;
			grid.RowHeadersWidths = new float[] { 10, 30 };
			grid.ColumnHeadersHeights = new float[] { 15 };
			grid.MinHeaderSize = 17;
			grid.AlignCells = true;

			for (int i = 0; i < 12; i++)
			{
				var header = new Header(new DateTime(DateTime.Today.Year, i + 1, 1).ToString("MMM yyyy"));
				header.TitleFont = headerFont;
				grid.ColumnHeaders.Add(header);
			}

			string[] g1Headers = new string[] {
           		"Activity A",
				"Activity B",
				"Activity C"
			};
			string[] g2Headers = new string[] {
				"Activity D",
				"Activity E",
				"Activity F"
           	};

			var g1Header = new Header("Group 1");
			g1Header.TitleFont = headerFont;
			g1Header.RotateTitle = true;
			grid.RowHeaders.Add(g1Header);
			for (int i = 0; i < g1Headers.Length; i++)
			{
				var header = new Header(g1Headers[i]);
				header.TitleFont = headerFont;
				g1Header.SubHeaders.Add(header);
			}

			var g2Header = new Header("Group 2");
			g2Header.TitleFont = headerFont;
			g2Header.RotateTitle = true;
			grid.RowHeaders.Add(g2Header);
			for (int i = 0; i < g2Headers.Length; i++)
			{
				var header = new Header(g2Headers[i]);
				header.TitleFont = headerFont;
				g2Header.SubHeaders.Add(header);
			}

			var cellStyle = grid[null, null].Style;
			cellStyle.LeftBorderPen = null;
			cellStyle.TopBorderPen = null;
			cellStyle.RightBorderPen = dashPen;
			cellStyle.BottomBorderPen = dashPen;

			cellStyle = grid[null, grid.RowHeaders[grid.RowHeaders.Count - 1]].Style;
			cellStyle.BottomBorderPen = borderPen;

			cellStyle = grid[grid.ColumnHeaders[grid.ColumnHeaders.Count - 1], null].Style;
			cellStyle.RightBorderPen = borderPen;

			for (int i = 0; i < grid.ColumnHeaders.Count; i++)
			{
				var header = grid.ColumnHeaders[i];
				if (i == grid.ColumnHeaders.Count - 1)
					header.Style.RightBorderPen = borderPen;
				else
					header.Style.RightBorderPen = whitePen;
				header.Style.TopBorderPen = borderPen;
				header.Style.LeftBorderPen = null;
				header.Style.BottomBorderPen = borderPen;
				header.Style.BackgroundBrush = new GradientBrush(secondaryColor, primaryColor, 90);
				header.TitleColor = Color.White;
			}

			for (int i = 0; i < grid.RowHeaders.Count; i++)
			{
				var header = grid.RowHeaders[i];
				header.Style.RightBorderPen = whitePen;
				header.Style.LeftBorderPen = borderPen;
				if (i == grid.RowHeaders.Count - 1)
					header.Style.BottomBorderPen = borderPen;
				else
					header.Style.BottomBorderPen = whitePen;
				header.Style.TopBorderPen = null;
				header.Style.BackgroundBrush = new SolidBrush(primaryColor);
				header.TitleColor = Color.White;

				for (int j = 0; j < header.SubHeaders.Count; j++)
				{
					var subheader = header.SubHeaders[j];
					subheader.Style.RightBorderPen = borderPen;
					subheader.Style.LeftBorderPen = null;
					if (i == grid.RowHeaders.Count - 1 && j == header.SubHeaders.Count - 1)
						subheader.Style.BottomBorderPen = borderPen;
					else
						subheader.Style.BottomBorderPen = whitePen;
					subheader.Style.TopBorderPen = null;
					subheader.Style.BackgroundBrush = new SolidBrush(primaryColor);
					subheader.TitleColor = Color.White;
				}
			}

			var rowStyle = new MindFusion.Diagramming.Lanes.Style();
			rowStyle.LeftBorderPen = borderPen;
			rowStyle.RightBorderPen = whitePen;
			rowStyle.BottomBorderPen = whitePen;
			rowStyle.TopBorderPen = borderPen;
			rowStyle.BackgroundBrush = new GradientBrush(secondaryColor, primaryColor, 90);
			grid.TopLeftAreaStyle = rowStyle;

			diagram.EnableLanes = true;
			diagram.BackBrush = new SolidBrush(Color.White);
			diagram.DefaultShape = Shape.FromId("Rectangle");

			var bounds = RectangleF.Union(diagram.LaneGrid.GetRowHeaderBounds(), diagram.LaneGrid.GetColumnHeaderBounds());
			diagram.Bounds = new RectangleF(0, 0, bounds.Right, bounds.Bottom);

			//Add some nodes and links
			diagram.Factory.CreateShapeNode(GetAreaBounds(diagram.LaneGrid, 0, 0, 4, 0));
			diagram.Factory.CreateShapeNode(GetAreaBounds(diagram.LaneGrid, 1, 1, 4, 1));
			diagram.Factory.CreateShapeNode(GetAreaBounds(diagram.LaneGrid, 2, 2, 4, 2));

			diagram.Factory.CreateShapeNode(GetAreaBounds(diagram.LaneGrid, 5, 3, 9, 3));
			diagram.Factory.CreateShapeNode(GetAreaBounds(diagram.LaneGrid, 5, 4, 11, 4));
			diagram.Factory.CreateShapeNode(GetAreaBounds(diagram.LaneGrid, 10, 5, 11, 5));

			// Customize the nodes some nodes
			var style1 = new ShapeNodeStyle();
			style1.Brush = new GradientBrush(Color.Green, Color.LawnGreen, 90);
			style1.Effects.Add(new AeroEffect());
			var style2 = new ShapeNodeStyle();
			style2.Brush = new GradientBrush(Color.Firebrick, Color.OrangeRed, 90);
			style2.Effects.Add(new AeroEffect());

			for (int i = 0; i < diagram.Nodes.Count; i++)
			{
				ShapeNode shapenode = (ShapeNode)diagram.Nodes[i];
				shapenode.Text = String.Format("Activity node {0}", i + 1);
				if (i < 3)
					shapenode.Style = style1;
				else
					shapenode.Style = style2;
			}

			// Create a default style for ShapeNodes 
			ShapeNodeStyle style = new ShapeNodeStyle();
			style.FontFamily = "Trebuchet MS";
			style.FontSize = 8.5f;
			style.FontStyle = FontStyle.Bold;
			style.Effects.Add(new AeroEffect());
			style.Stroke = new SolidBrush(Color.DimGray);
			style.TextBrush = new SolidBrush(Color.FromArgb(0x30, 0x30, 0x30));
			style.ShadowBrush = new SolidBrush(Color.Transparent);
			style.Brush = new GradientBrush(Color.OrangeRed, Color.Orange, 90);
			Theme theme = new Theme();
			theme.RegisterStyle(typeof(ShapeNode), style);
			diagram.Theme = theme;
		}
	}

	private RectangleF GetAreaBounds(Grid grid, int left, int top, int right, int bottom)
	{
		return GetAreaBounds(grid.GetCellBounds(left, top), grid.GetCellBounds(right, bottom));
	}

	private RectangleF GetAreaBounds(RectangleF bounds1, RectangleF bounds2)
	{
		var bounds = RectangleF.Union(bounds1, bounds2);
		bounds.Inflate(-2, -3);
		bounds.Width -= 1;
		bounds.Height -= 1;
		return bounds;
	}

	static int n = 1;

	protected void diagramView_NodeCreated(object sender, NodeEventArgs e)
	{
		ShapeNode shape = e.Node as ShapeNode;
		if (shape != null)
			shape.Text = String.Format("New node {0}", n);

		TableNode table = e.Node as TableNode;
		if (table != null)
			table.Caption = String.Format("New node {0}", n);

		n += 1;
		lblStatus.Text = String.Format("New node created at X={0},Y={1}", e.Node.Bounds.X, e.Node.Bounds.Y);

		if (shape != null)
			UpdateNodeBounds(shape);
	}

	private void UpdateNodeBounds(DiagramNode node)
	{
		// Align the node to the nearest cells
		RectangleF cell1Bounds = RectangleF.Empty;
		RectangleF cell2Bounds = RectangleF.Empty;
		var grid = node.Parent.LaneGrid;
		if (grid.GetCellFromPoint(new PointF(node.Bounds.Left, node.Bounds.Top), ref cell1Bounds) != null &&
			grid.GetCellFromPoint(new PointF(node.Bounds.Right, node.Bounds.Top), ref cell2Bounds) != null)
		{
			node.Bounds = GetAreaBounds(cell1Bounds, cell2Bounds);
		}
	}

	private static string GetLabel(DiagramNode node)
	{
		string label = "";

		ShapeNode shape = node as ShapeNode;
		if (shape != null)
			label = shape.Text;

		TableNode table = node as TableNode;
		if (table != null)
			label = table.Caption;

		return label;
	}

	protected void diagramView_NodeDeleted(object sender, NodeEventArgs e)
	{
		lblStatus.Text = String.Format("Node '{0}' deleted", GetLabel(e.Node));
	}

	protected void diagramView_NodeModified(object sender, NodeEventArgs e)
	{
		lblStatus.Text = String.Format(
			"Node '{0}' modified by {1} handle",
			GetLabel(e.Node), e.AdjustmentHandle);

		if (e.Node is ShapeNode)
			UpdateNodeBounds(e.Node);
	}

	protected void diagramView_NodeClicked(object sender, NodeEventArgs e)
	{
		lblStatus.Text = String.Format("Node '{0}' clicked", GetLabel(e.Node));
	}

	protected void behavior_SelectedIndexChanged(object sender, EventArgs e)
	{
		switch (behavior.SelectedIndex)
		{
			case 0:
				diagramView.Behavior = Behavior.Modify;
				break;
			case 1:
				diagramView.Behavior = Behavior.DrawShapes;
				break;
			case 2:
				diagramView.Behavior = Behavior.LinkShapes;
				break;
			case 3:
				diagramView.Behavior = Behavior.DrawTables;
				break;
			case 4:
				diagramView.Behavior = Behavior.LinkTables;
				break;
			case 5:
				diagramView.Behavior = Behavior.DrawLinks;
				break;
			case 6:
				diagramView.Behavior = Behavior.DoNothing;
				break;
		}
	}
}